@@ -2,6 +2,7 @@ import sys
 import time
 import os
 import json
+import stat
 
 from PyQt5 import QtCore, QtGui, uic, QtWidgets
 from PyQt5.QtCore import QCoreApplication as QC
@@ -9,10 +10,11 @@ from PyQt5.QtCore import QCoreApplication as QC
 from opensnitch.config import Config
 from opensnitch.nodes import Nodes
 from opensnitch.database import Database
-from opensnitch.utils import Message, QuickHelp, Themes
+from opensnitch.utils import Message, QuickHelp, Themes, Icons, languages
+from opensnitch.utils.xdg import Autostart
 from opensnitch.notifications import DesktopNotifications
 
-from opensnitch import ui_pb2
+from opensnitch import ui_pb2, auth
 
 DIALOG_UI_PATH = "%s/../res/preferences.ui" % os.path.dirname(sys.modules[__name__].__file__)
 class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
@@ -23,21 +25,29 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
     TAB_POPUPS = 0
     TAB_UI = 1
-    TAB_NODES = 2
-    TAB_DB = 3
+    TAB_RULES = 2
+    TAB_NODES = 3
+    TAB_DB = 4
 
     SUM = 1
     REST = 0
 
+    AUTH_SIMPLE = 0
+    AUTH_TLS_SIMPLE = 1
+    AUTH_TLS_MUTUAL = 2
+
     def __init__(self, parent=None, appicon=None):
         QtWidgets.QDialog.__init__(self, parent, QtCore.Qt.WindowStaysOnTopHint)
 
         self._themes = Themes.instance()
         self._saved_theme = ""
+        self._restart_msg = QC.translate("preferences", "Restart the GUI in order changes to take effect")
+        self._changes_needs_restart = None
 
         self._cfg = Config.get()
         self._nodes = Nodes.instance()
         self._db = Database.instance()
+        self._autostart = Autostart()
 
         self._notification_callback.connect(self._cb_notification_callback)
         self._notifications_sent = {}
@@ -57,6 +67,7 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.popupsCheck.clicked.connect(self._cb_popups_check_toggled)
         self.dbFileButton.clicked.connect(self._cb_file_db_clicked)
         self.checkUIRules.toggled.connect(self._cb_check_ui_rules_toggled)
+        self.comboUITheme.currentIndexChanged.connect(self._cb_combo_themes_changed)
         self.cmdTimeoutUp.clicked.connect(lambda: self._cb_cmd_spin_clicked(self.spinUITimeout, self.SUM))
         self.cmdTimeoutDown.clicked.connect(lambda: self._cb_cmd_spin_clicked(self.spinUITimeout, self.REST))
         self.cmdDBMaxDaysUp.clicked.connect(lambda: self._cb_cmd_spin_clicked(self.spinDBMaxDays, self.SUM))
@@ -67,37 +78,74 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.radioSysNotifs.clicked.connect(self._cb_radio_system_notifications)
         self.helpButton.setToolTipDuration(30 * 1000)
 
-        if QtGui.QIcon.hasThemeIcon("emblem-default") == False:
-            self.applyButton.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_DialogApplyButton")))
-            self.cancelButton.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_DialogCloseButton")))
-            self.acceptButton.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_DialogSaveButton")))
-            self.dbFileButton.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_DirOpenIcon")))
+        self.comboAuthType.currentIndexChanged.connect(self._cb_combo_auth_type_changed)
+        self.comboAuthType.setItemData(PreferencesDialog.AUTH_SIMPLE, auth.Simple)
+        self.comboAuthType.setItemData(PreferencesDialog.AUTH_TLS_SIMPLE, auth.TLSSimple)
+        self.comboAuthType.setItemData(PreferencesDialog.AUTH_TLS_MUTUAL, auth.TLSMutual)
 
-        if QtGui.QIcon.hasThemeIcon("list-add") == False:
-            self.cmdTimeoutUp.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_ArrowUp")))
-            self.cmdTimeoutDown.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_ArrowDown")))
-            self.cmdDBMaxDaysUp.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_ArrowUp")))
-            self.cmdDBMaxDaysDown.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_ArrowDown")))
-            self.cmdDBPurgesUp.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_ArrowUp")))
-            self.cmdDBPurgesDown.setIcon(self.style().standardIcon(getattr(QtWidgets.QStyle, "SP_ArrowDown")))
+        self.comboUIRules.currentIndexChanged.connect(self._cb_combo_uirules_changed)
+
+        if QtGui.QIcon.hasThemeIcon("emblem-default"):
+            return
+
+        saveIcon = Icons.new(self, "document-save")
+        applyIcon = Icons.new(self, "emblem-default")
+        delIcon = Icons.new(self, "edit-delete")
+        closeIcon = Icons.new(self, "window-close")
+        openIcon = Icons.new(self, "document-open")
+        helpIcon = Icons.new(self, "help-browser")
+        addIcon = Icons.new(self, "list-add")
+        delIcon = Icons.new(self, "list-remove")
+        allowIcon = Icons.new(self, "emblem-default")
+        denyIcon = Icons.new(self, "emblem-important")
+        rejectIcon = Icons.new(self, "window-close")
+        self.applyButton.setIcon(applyIcon)
+        self.cancelButton.setIcon(closeIcon)
+        self.acceptButton.setIcon(saveIcon)
+        self.helpButton.setIcon(helpIcon)
+        self.dbFileButton.setIcon(openIcon)
+
+        self.cmdTimeoutUp.setIcon(addIcon)
+        self.cmdTimeoutDown.setIcon(delIcon)
+        self.cmdDBMaxDaysUp.setIcon(addIcon)
+        self.cmdDBMaxDaysDown.setIcon(delIcon)
+        self.cmdDBPurgesUp.setIcon(addIcon)
+        self.cmdDBPurgesDown.setIcon(delIcon)
+
+        self.comboUIAction.setItemIcon(Config.ACTION_DENY_IDX, denyIcon)
+        self.comboUIAction.setItemIcon(Config.ACTION_ALLOW_IDX, allowIcon)
+        self.comboUIAction.setItemIcon(Config.ACTION_REJECT_IDX, rejectIcon)
 
     def showEvent(self, event):
         super(PreferencesDialog, self).showEvent(event)
 
         try:
+            self._changes_needs_restart = None
             self._settingsSaved = False
             self._reset_status_message()
             self._hide_status_label()
             self.comboNodes.clear()
 
+            self._load_langs()
+
+            self.comboNodeAddress.clear()
+            run_path = "/run/user/{0}/opensnitch/".format(os.getuid())
+            var_run_path = "/var{0}".format(run_path)
+            self.comboNodeAddress.addItem("unix:///tmp/osui.sock")
+            if os.path.exists(run_path):
+                self.comboNodeAddress.addItem("unix://%s/osui.sock" % run_path)
+            if os.path.exists(var_run_path):
+                self.comboNodeAddress.addItem("unix://%s/osui.sock" % var_run_path)
+
             self._node_list = self._nodes.get()
             for addr in self._node_list:
                 self.comboNodes.addItem(addr)
 
             if len(self._node_list) == 0:
                 self._reset_node_settings()
+                self._set_status_message(QC.translate("preferences", "There're no nodes connected"))
         except Exception as e:
-            print(self.LOG_TAG + "exception loading nodes", e)
+            print(self.LOG_TAG + "exception loading nodes:", e)
 
         self._load_settings()
 
@@ -109,16 +157,39 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.comboNodeMonitorMethod.currentIndexChanged.connect(self._cb_node_needs_update)
         self.comboNodeLogLevel.currentIndexChanged.connect(self._cb_node_needs_update)
         self.comboNodeLogFile.currentIndexChanged.connect(self._cb_node_needs_update)
+        self.checkNodeLogUTC.clicked.connect(self._cb_node_needs_update)
+        self.checkNodeLogMicro.clicked.connect(self._cb_node_needs_update)
         self.comboNodeAddress.currentTextChanged.connect(self._cb_node_needs_update)
         self.checkInterceptUnknown.clicked.connect(self._cb_node_needs_update)
         self.checkApplyToNodes.clicked.connect(self._cb_node_needs_update)
         self.comboDBType.currentIndexChanged.connect(self._cb_db_type_changed)
         self.checkDBMaxDays.toggled.connect(self._cb_db_max_days_toggled)
+        self.lineCACertFile.textChanged.connect(self._cb_line_certs_changed)
+        self.lineCertFile.textChanged.connect(self._cb_line_certs_changed)
+        self.lineCertKeyFile.textChanged.connect(self._cb_line_certs_changed)
 
         # True when any node option changes
         self._node_needs_update = False
 
+    def show_node_prefs(self, addr):
+        self.show()
+        self.comboNodes.setCurrentText(addr)
+        self.tabWidget.setCurrentIndex(self.TAB_NODES)
+
+    def _load_langs(self):
+        try:
+            self.comboUILang.clear()
+            self.comboUILang.blockSignals(True)
+            self.comboUILang.addItem(QC.translate("preferences", "System default"), "")
+            langs, langNames = languages.get_all()
+            for idx, lang in enumerate(langs):
+                self.comboUILang.addItem(langNames[idx].capitalize(), langs[idx])
+            self.comboUILang.blockSignals(False)
+        except Exception as e:
+            print(self.LOG_TAG + "exception loading languages:", e)
+
     def _load_themes(self):
+        self.comboUITheme.blockSignals(True)
         theme_idx, self._saved_theme = self._themes.get_saved_theme()
 
         self.labelThemeError.setVisible(False)
@@ -135,11 +206,12 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self.labelThemeError.setText(QC.translate("preferences", "Themes not available. Install qt-material: pip3 install qt-material"))
 
         self.comboUITheme.setCurrentIndex(theme_idx)
+        self.comboUITheme.blockSignals(False)
 
     def _load_settings(self):
         self._default_action = self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY)
         self._default_target = self._cfg.getInt(self._cfg.DEFAULT_TARGET_KEY, 0)
-        self._default_timeout = self._cfg.getInt(self._cfg.DEFAULT_TIMEOUT_KEY, 15)
+        self._default_timeout = self._cfg.getInt(self._cfg.DEFAULT_TIMEOUT_KEY, Config.DEFAULT_TIMEOUT)
         self._disable_popups = self._cfg.getBool(self._cfg.DEFAULT_DISABLE_POPUPS)
 
         if self._cfg.hasKey(self._cfg.DEFAULT_DURATION_KEY):
@@ -150,6 +222,33 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.comboUIDuration.setCurrentIndex(self._default_duration)
         self.comboUIDialogPos.setCurrentIndex(self._cfg.getInt(self._cfg.DEFAULT_POPUP_POSITION))
 
+        self.checkAutostart.setChecked(self._autostart.isEnabled())
+
+        maxmsgsize = self._cfg.getSettings(Config.DEFAULT_SERVER_MAX_MESSAGE_LENGTH)
+        if maxmsgsize:
+            self.comboGrpcMsgSize.setCurrentText(maxmsgsize)
+        else:
+            self.comboGrpcMsgSize.setCurrentIndex(0)
+
+        self.lineCACertFile.setText(self._cfg.getSettings(Config.AUTH_CA_CERT))
+        self.lineCertFile.setText(self._cfg.getSettings(Config.AUTH_CERT))
+        self.lineCertKeyFile.setText(self._cfg.getSettings(Config.AUTH_CERTKEY))
+        authtype_idx = self.comboAuthType.findData(self._cfg.getSettings(Config.AUTH_TYPE))
+        if authtype_idx <= 0:
+            authtype_idx = 0
+            self.lineCACertFile.setEnabled(False)
+            self.lineCertFile.setEnabled(False)
+            self.lineCertKeyFile.setEnabled(False)
+        self.comboAuthType.setCurrentIndex(authtype_idx)
+
+        saved_lang = self._cfg.getSettings(Config.DEFAULT_LANGUAGE)
+        if saved_lang:
+            saved_langname = self._cfg.getSettings(Config.DEFAULT_LANGNAME)
+            self.comboUILang.blockSignals(True)
+            self.comboUILang.setCurrentText(saved_langname)
+            self.comboUILang.blockSignals(False)
+
+        self.comboUIRules.blockSignals(True)
         self.comboUIRules.setCurrentIndex(self._cfg.getInt(self._cfg.DEFAULT_IGNORE_TEMPORARY_RULES))
         self.checkUIRules.setChecked(self._cfg.getBool(self._cfg.DEFAULT_IGNORE_RULES))
         self.comboUIRules.setEnabled(self._cfg.getBool(self._cfg.DEFAULT_IGNORE_RULES))
@@ -159,6 +258,7 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self._cfg.getBool(self._cfg.DEFAULT_IGNORE_RULES),
             self._cfg.getInt(self._cfg.DEFAULT_IGNORE_TEMPORARY_RULES)
         )
+        self.comboUIRules.blockSignals(False)
 
         self.comboUIAction.setCurrentIndex(self._default_action)
         self.comboUITarget.setCurrentIndex(self._default_target)
@@ -212,6 +312,13 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 self.checkInterceptUnknown.setChecked(node_config['InterceptUnknown'])
                 self.comboNodeLogLevel.setCurrentIndex(int(node_config['LogLevel']))
 
+                if node_config.get('LogUTC') == None:
+                    node_config['LogUTC'] = False
+                self.checkNodeLogUTC.setChecked(node_config['LogUTC'])
+                if node_config.get('LogMicro') == None:
+                    node_config['LogMicro'] = False
+                self.checkNodeLogMicro.setChecked(node_config['LogMicro'])
+
                 if node_config.get('Server') != None:
                     self.comboNodeAddress.setEnabled(True)
                     self.comboNodeLogFile.setEnabled(True)
@@ -244,12 +351,13 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             node_config['DefaultDuration'] = node_duration
             node_config['ProcMonitorMethod'] = self.comboNodeMonitorMethod.currentText()
             node_config['LogLevel'] = self.comboNodeLogLevel.currentIndex()
+            node_config['LogUTC'] = self.checkNodeLogUTC.isChecked()
+            node_config['LogMicro'] = self.checkNodeLogMicro.isChecked()
             node_config['InterceptUnknown'] = self.checkInterceptUnknown.isChecked()
 
             if node_config.get('Server') != None:
                 # skip setting Server Address if we're applying the config to all nodes
-                if self.checkApplyToNodes.isChecked():
-                    node_config['Server']['Address'] = self.comboNodeAddress.currentText()
+                node_config['Server']['Address'] = self.comboNodeAddress.currentText()
                 node_config['Server']['LogFile'] = self.comboNodeLogFile.currentText()
             else:
                 print(addr, " doesn't have Server item")
@@ -264,8 +372,9 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         if cols == None:
             return
 
-        for c in range(7):
+        for c in range(8):
             checked = str(c) in cols
+
             if c == 0:
                 self.checkHideTime.setChecked(checked)
             elif c == 1:
@@ -279,6 +388,8 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             elif c == 5:
                 self.checkHideProc.setChecked(checked)
             elif c == 6:
+                self.checkHideCmdline.setChecked(checked)
+            elif c == 7:
                 self.checkHideRule.setChecked(checked)
 
     def _reset_node_settings(self):
@@ -287,12 +398,16 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.comboNodeMonitorMethod.setCurrentIndex(0)
         self.checkInterceptUnknown.setChecked(False)
         self.comboNodeLogLevel.setCurrentIndex(0)
+        self.checkNodeLogUTC.setChecked(True)
+        self.checkNodeLogMicro.setChecked(False)
         self.labelNodeName.setText("")
         self.labelNodeVersion.setText("")
 
     def _save_settings(self):
+        self._reset_status_message()
         self._save_ui_config()
-        self._save_db_config()
+        if not self._save_db_config():
+            return
 
         if self.tabWidget.currentIndex() == self.TAB_NODES:
             self._show_status_label()
@@ -319,24 +434,22 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 except Exception as e:
                     print(self.LOG_TAG + "exception saving config: ", e)
                     self._set_status_error(QC.translate("preferences", "Exception saving config: {0}").format(str(e)))
+            elif addr == "":
+                self._set_status_message(QC.translate("preferences", "There're no nodes connected"))
 
             self._node_needs_update = False
 
         self.saved.emit()
         self._settingsSaved = True
+        self._needs_restart()
 
     def _save_db_config(self):
         dbtype = self.comboDBType.currentIndex()
-        self._cfg.setSettings(Config.DEFAULT_DB_TYPE_KEY, dbtype)
-        self._cfg.setSettings(Config.DEFAULT_DB_PURGE_OLDEST, bool(self.checkDBMaxDays.isChecked()))
-        self._cfg.setSettings(Config.DEFAULT_DB_MAX_DAYS, int(self.spinDBMaxDays.value()))
-        self._cfg.setSettings(Config.DEFAULT_DB_PURGE_INTERVAL, int(self.spinDBPurgeInterval.value()))
+        db_name = self._cfg.getSettings(self._cfg.DEFAULT_DB_FILE_KEY)
 
-        if self.comboDBType.currentIndex() == self.dbType:
-            return
-
-        if dbtype == self._db.get_db_file():
-            return
+        if self.dbLabel.text() != "" and \
+                (self.comboDBType.currentIndex() != self.dbType or db_name != self.dbLabel.text()):
+            self._changes_needs_restart = QC.translate("preferences", "DB type changed")
 
         if self.comboDBType.currentIndex() != Database.DB_TYPE_MEMORY:
             if self.dbLabel.text() != "":
@@ -346,53 +459,89 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                     QC.translate("preferences", "Warning"),
                     QC.translate("preferences", "You must select a file for the database<br>or choose \"In memory\" type."),
                     QtWidgets.QMessageBox.Warning)
-                return
-
-        Message.ok(
-            QC.translate("preferences", "DB type changed"),
-            QC.translate("preferences", "Restart the GUI in order effects to take effect"),
-            QtWidgets.QMessageBox.Warning)
+                self.dbLabel.setText("")
+                return False
 
+        self._cfg.setSettings(Config.DEFAULT_DB_TYPE_KEY, dbtype)
+        self._cfg.setSettings(Config.DEFAULT_DB_PURGE_OLDEST, bool(self.checkDBMaxDays.isChecked()))
+        self._cfg.setSettings(Config.DEFAULT_DB_MAX_DAYS, int(self.spinDBMaxDays.value()))
+        self._cfg.setSettings(Config.DEFAULT_DB_PURGE_INTERVAL, int(self.spinDBPurgeInterval.value()))
         self.dbType = self.comboDBType.currentIndex()
 
+        return True
+
     def _save_ui_config(self):
-        self._save_ui_columns_config()
+        try:
+            self._save_ui_columns_config()
 
-        self._cfg.setSettings(self._cfg.DEFAULT_IGNORE_TEMPORARY_RULES, int(self.comboUIRules.currentIndex()))
-        self._cfg.setSettings(self._cfg.DEFAULT_IGNORE_RULES, bool(self.checkUIRules.isChecked()))
-        #self._set_rules_duration_filter()
-        self._cfg.setRulesDurationFilter(
-            bool(self.checkUIRules.isChecked()),
-            int(self.comboUIRules.currentIndex())
-        )
+            maxmsgsize = self.comboGrpcMsgSize.currentText()
+            if maxmsgsize != "":
+                self._cfg.setSettings(Config.DEFAULT_SERVER_MAX_MESSAGE_LENGTH, maxmsgsize.replace(" ", ""))
 
-        self._cfg.setSettings(self._cfg.DEFAULT_ACTION_KEY, self.comboUIAction.currentIndex())
-        self._cfg.setSettings(self._cfg.DEFAULT_DURATION_KEY, int(self.comboUIDuration.currentIndex()))
-        self._cfg.setSettings(self._cfg.DEFAULT_TARGET_KEY, self.comboUITarget.currentIndex())
-        self._cfg.setSettings(self._cfg.DEFAULT_TIMEOUT_KEY, self.spinUITimeout.value())
-        self._cfg.setSettings(self._cfg.DEFAULT_DISABLE_POPUPS, bool(self.popupsCheck.isChecked()))
-        self._cfg.setSettings(self._cfg.DEFAULT_POPUP_POSITION, int(self.comboUIDialogPos.currentIndex()))
+            savedauthtype = self._cfg.getSettings(Config.AUTH_TYPE)
+            authtype = self.comboAuthType.itemData(self.comboAuthType.currentIndex())
+            cacert = self._cfg.getSettings(Config.AUTH_CA_CERT)
+            cert = self._cfg.getSettings(Config.AUTH_CERT)
+            certkey = self._cfg.getSettings(Config.AUTH_CERTKEY)
+            if not self._validate_certs():
+                return
 
-        self._cfg.setSettings(self._cfg.DEFAULT_POPUP_ADVANCED, bool(self.showAdvancedCheck.isChecked()))
-        self._cfg.setSettings(self._cfg.DEFAULT_POPUP_ADVANCED_DSTIP, bool(self.dstIPCheck.isChecked()))
-        self._cfg.setSettings(self._cfg.DEFAULT_POPUP_ADVANCED_DSTPORT, bool(self.dstPortCheck.isChecked()))
-        self._cfg.setSettings(self._cfg.DEFAULT_POPUP_ADVANCED_UID, bool(self.uidCheck.isChecked()))
+            if savedauthtype != authtype or self.lineCertFile.text() != cert or \
+                    self.lineCertKeyFile.text() != certkey or self.lineCACertFile.text() != cacert:
+                self._changes_needs_restart = QC.translate("preferences", "Certificates changed")
+            self._cfg.setSettings(Config.AUTH_TYPE, authtype)
+            self._cfg.setSettings(Config.AUTH_CA_CERT, self.lineCACertFile.text())
+            self._cfg.setSettings(Config.AUTH_CERT, self.lineCertFile.text())
+            self._cfg.setSettings(Config.AUTH_CERTKEY, self.lineCertKeyFile.text())
 
-        self._cfg.setSettings(self._cfg.NOTIFICATIONS_ENABLED, bool(self.groupNotifs.isChecked()))
-        self._cfg.setSettings(self._cfg.NOTIFICATIONS_TYPE,
-                              int(Config.NOTIFICATION_TYPE_SYSTEM if self.radioSysNotifs.isChecked() else Config.NOTIFICATION_TYPE_QT))
+            selected_lang = self.comboUILang.itemData(self.comboUILang.currentIndex())
+            saved_lang = self._cfg.getSettings(Config.DEFAULT_LANGUAGE)
+            saved_lang = "" if saved_lang is None else saved_lang
+            if saved_lang != selected_lang:
+                languages.save(self._cfg, selected_lang)
+                self._changes_needs_restart = QC.translate("preferences", "Language changed")
 
-        self._themes.save_theme(self.comboUITheme.currentIndex(), self.comboUITheme.currentText())
-        if self._themes.available() and self._saved_theme != self.comboUITheme.currentText():
-            Message.ok(
-                QC.translate("preferences", "UI theme changed"),
-                QC.translate("preferences", "Restart the GUI in order to apply the new theme"),
-                QtWidgets.QMessageBox.Warning)
+            self._cfg.setSettings(self._cfg.DEFAULT_IGNORE_TEMPORARY_RULES, int(self.comboUIRules.currentIndex()))
+            self._cfg.setSettings(self._cfg.DEFAULT_IGNORE_RULES, bool(self.checkUIRules.isChecked()))
+            #self._set_rules_duration_filter()
+            self._cfg.setRulesDurationFilter(
+                bool(self.checkUIRules.isChecked()),
+                int(self.comboUIRules.currentIndex())
+            )
+            if self.checkUIRules.isChecked():
+                self._nodes.delete_rule_by_field(Config.DURATION_FIELD, Config.RULES_DURATION_FILTER)
 
-        # this is a workaround for not display pop-ups.
-        # see #79 for more information.
-        if self.popupsCheck.isChecked():
-            self._cfg.setSettings(self._cfg.DEFAULT_TIMEOUT_KEY, 0)
+            self._cfg.setSettings(self._cfg.DEFAULT_ACTION_KEY, self.comboUIAction.currentIndex())
+            self._cfg.setSettings(self._cfg.DEFAULT_DURATION_KEY, int(self.comboUIDuration.currentIndex()))
+            self._cfg.setSettings(self._cfg.DEFAULT_TARGET_KEY, self.comboUITarget.currentIndex())
+            self._cfg.setSettings(self._cfg.DEFAULT_TIMEOUT_KEY, self.spinUITimeout.value())
+            self._cfg.setSettings(self._cfg.DEFAULT_DISABLE_POPUPS, bool(self.popupsCheck.isChecked()))
+            self._cfg.setSettings(self._cfg.DEFAULT_POPUP_POSITION, int(self.comboUIDialogPos.currentIndex()))
+
+            self._cfg.setSettings(self._cfg.DEFAULT_POPUP_ADVANCED, bool(self.showAdvancedCheck.isChecked()))
+            self._cfg.setSettings(self._cfg.DEFAULT_POPUP_ADVANCED_DSTIP, bool(self.dstIPCheck.isChecked()))
+            self._cfg.setSettings(self._cfg.DEFAULT_POPUP_ADVANCED_DSTPORT, bool(self.dstPortCheck.isChecked()))
+            self._cfg.setSettings(self._cfg.DEFAULT_POPUP_ADVANCED_UID, bool(self.uidCheck.isChecked()))
+
+            self._cfg.setSettings(self._cfg.NOTIFICATIONS_ENABLED, bool(self.groupNotifs.isChecked()))
+            self._cfg.setSettings(self._cfg.NOTIFICATIONS_TYPE,
+                                int(Config.NOTIFICATION_TYPE_SYSTEM if self.radioSysNotifs.isChecked() else Config.NOTIFICATION_TYPE_QT))
+
+            self._themes.save_theme(self.comboUITheme.currentIndex(), self.comboUITheme.currentText())
+
+            if self._themes.available() and self._saved_theme != "" and self.comboUITheme.currentText() == QC.translate("preferences", "System"):
+                self._changes_needs_restart = QC.translate("preferences", "UI theme changed")
+
+            # this is a workaround for not display pop-ups.
+            # see #79 for more information.
+            if self.popupsCheck.isChecked():
+                self._cfg.setSettings(self._cfg.DEFAULT_TIMEOUT_KEY, 0)
+
+
+            self._autostart.enable(self.checkAutostart.isChecked())
+
+        except Exception as e:
+            self._set_status_error(str(e))
 
     def _save_ui_columns_config(self):
         cols=list()
@@ -408,8 +557,10 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             cols.append("4")
         if self.checkHideProc.isChecked():
             cols.append("5")
-        if self.checkHideRule.isChecked():
+        if self.checkHideCmdline.isChecked():
             cols.append("6")
+        if self.checkHideRule.isChecked():
+            cols.append("7")
 
         self._cfg.setSettings(Config.STATS_SHOW_COLUMNS, cols)
 
@@ -420,20 +571,60 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             if error != None:
                 return error
 
-            if addr.startswith("unix://"):
-                self._cfg.setSettings(self._cfg.DEFAULT_DEFAULT_SERVER_ADDR, self.comboNodeAddress.currentText())
-            else:
-                self._nodes.save_node_config(addr, notifObject.data)
-                nid = self._nodes.send_notification(addr, notifObject, self._notification_callback)
+            savedAddr = self._cfg.getSettings(Config.DEFAULT_SERVER_ADDR)
+            # exclude this message if there're more than one node connected
+            if self.comboNodes.count() == 1 and savedAddr != None and savedAddr != self.comboNodeAddress.currentText():
+                self._changes_needs_restart = QC.translate("preferences", "Ok")
+
+            self._cfg.setSettings(Config.DEFAULT_SERVER_ADDR, self.comboNodeAddress.currentText())
+
+            self._nodes.save_node_config(addr, notifObject.data)
+            nid = self._nodes.send_notification(addr, notifObject, self._notification_callback)
+            self._notifications_sent[nid] = notifObject
 
-                self._notifications_sent[nid] = notifObject
         except Exception as e:
             print(self.LOG_TAG + "exception saving node config on %s: " % addr, e)
-            self._set_status_error(QC.translate("Exception saving node config {0}: {1}").format((addr, str(e))))
+            self._set_status_error(QC.translate("preferences", "Exception saving node config {0}: {1}").format((addr, str(e))))
             return addr + ": " + str(e)
 
         return None
 
+    def _validate_certs(self):
+        try:
+            if self.comboAuthType.currentIndex() == PreferencesDialog.AUTH_SIMPLE:
+                return True
+
+            if self.comboAuthType.currentIndex() > 0 and (self.lineCertFile.text() == "" or self.lineCertKeyFile.text() == ""):
+                raise ValueError(QC.translate("preferences", "Certs fields cannot be empty."))
+
+            if oct(stat.S_IMODE(os.lstat(self.lineCertFile.text()).st_mode)) != "0o600":
+                self._set_status_message(
+                    QC.translate("preferences", "cert file has excessive permissions, it should have 0600")
+                )
+            if oct(stat.S_IMODE(os.lstat(self.lineCertFile.text()).st_mode)) != "0o600":
+                self._set_status_message(
+                    QC.translate("preferences", "cert key file has excessive permissions, it should have 0600")
+                )
+
+            if self.comboAuthType.currentIndex() == PreferencesDialog.AUTH_TLS_MUTUAL:
+                if oct(stat.S_IMODE(os.lstat(self.lineCACertFile.text()).st_mode)) != "0o600":
+                    self._set_status_message(
+                        QC.translate("preferences", "CA cert file has excessive permissions, it should have 0600")
+                    )
+
+            return True
+        except Exception as e:
+            self._changes_needs_restart = None
+            self._set_status_error("certs error: {0}".format(e))
+            return False
+
+    def _needs_restart(self):
+        if self._changes_needs_restart:
+            Message.ok(self._changes_needs_restart,
+                self._restart_msg,
+                QtWidgets.QMessageBox.Warning)
+            self._changes_needs_restart = None
+
     def _hide_status_label(self):
         self.statusLabel.hide()
 
@@ -461,9 +652,12 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
     def _enable_db_cleaner_options(self, enable, db_max_days):
         self.checkDBMaxDays.setChecked(enable)
+        self.checkDBMaxDays.setEnabled(enable)
         self.spinDBMaxDays.setEnabled(enable)
         self.spinDBPurgeInterval.setEnabled(enable)
         self.labelDBPurgeInterval.setEnabled(enable)
+        self.labelDBPurgeDays.setEnabled(enable)
+        self.labelDBPurgeMinutes.setEnabled(enable)
         self.cmdDBMaxDaysUp.setEnabled(enable)
         self.cmdDBMaxDaysDown.setEnabled(enable)
         self.cmdDBPurgesUp.setEnabled(enable)
@@ -480,19 +674,29 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
             del self._notifications_sent[reply.id]
 
+    def _cb_line_certs_changed(self, text):
+        self._changes_needs_restart = QC.translate("preferences", "Certs changed")
+
     def _cb_file_db_clicked(self):
         options = QtWidgets.QFileDialog.Options()
         fileName, _ = QtWidgets.QFileDialog.getSaveFileName(self, "", "","All Files (*)", options=options)
         if fileName:
             self.dbLabel.setText(fileName)
 
+    def _cb_combo_uirules_changed(self, idx):
+        self._cfg.setRulesDurationFilter(
+            self._cfg.getBool(self._cfg.DEFAULT_IGNORE_RULES),
+            idx
+            #self._cfg.getInt(self._cfg.DEFAULT_IGNORE_TEMPORARY_RULES)
+        )
+
+
     def _cb_db_type_changed(self):
-        if self.comboDBType.currentIndex() == Database.DB_TYPE_MEMORY:
-            self.dbFileButton.setVisible(False)
-            self.dbLabel.setVisible(False)
-        else:
-            self.dbFileButton.setVisible(True)
-            self.dbLabel.setVisible(True)
+        isDBMem = self.comboDBType.currentIndex() == Database.DB_TYPE_MEMORY
+        self.dbFileButton.setVisible(not isDBMem)
+        self.dbLabel.setVisible(not isDBMem)
+        self.checkDBMaxDays.setEnabled(not isDBMem)
+        self.checkDBMaxDays.setChecked(not isDBMem)
 
     def _cb_accept_button_clicked(self):
         self.accept()
@@ -500,6 +704,7 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self._save_settings()
 
     def _cb_apply_button_clicked(self):
+        self._reset_status_message()
         self._save_settings()
 
     def _cb_cancel_button_clicked(self):
@@ -515,7 +720,7 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     def _cb_popups_check_toggled(self, checked):
         self.spinUITimeout.setEnabled(not checked)
         if not checked:
-            self.spinUITimeout.setValue(15)
+            self.spinUITimeout.setValue(20)
 
     def _cb_node_combo_changed(self, index):
         self._load_node_settings()
@@ -526,6 +731,19 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     def _cb_check_ui_rules_toggled(self, state):
         self.comboUIRules.setEnabled(state)
 
+    def _cb_combo_themes_changed(self, index):
+        self._themes.change_theme(self, self.comboUITheme.currentText())
+
+    def _cb_combo_auth_type_changed(self, index):
+        curtype = self.comboAuthType.itemData(self.comboAuthType.currentIndex())
+        savedtype = self._cfg.getSettings(Config.AUTH_TYPE)
+        if curtype != savedtype:
+            self._changes_needs_restart = QC.translate("preferences", "Auth type changed")
+
+        self.lineCACertFile.setEnabled(index == PreferencesDialog.AUTH_TLS_MUTUAL)
+        self.lineCertFile.setEnabled(index >= PreferencesDialog.AUTH_TLS_SIMPLE)
+        self.lineCertKeyFile.setEnabled(index >= PreferencesDialog.AUTH_TLS_SIMPLE)
+
     def _cb_db_max_days_toggled(self, state):
         self._enable_db_cleaner_options(state, 1)
 
@@ -535,6 +753,10 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         else:
             spinWidget.setValue(spinWidget.value() - 1)
 
+        enablePopups = spinWidget.value() > 0
+        self.popupsCheck.setChecked(not enablePopups)
+        self.spinUITimeout.setEnabled(enablePopups)
+
     def _cb_radio_system_notifications(self):
         if self._desktop_notifications.is_available() == False:
             self.radioSysNotifs.setChecked(False)
@@ -543,11 +765,17 @@ class PreferencesDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             return
 
     def _cb_test_notifs_clicked(self):
-        if self._desktop_notifications.is_available() == False:
-            self._set_status_error(QC.translate("notifications", "System notifications are not available, you need to install python3-notify2."))
-            return
+        try:
+            self.cmdTestNotifs.setEnabled(False)
+            if self._desktop_notifications.is_available() == False:
+                self._set_status_error(QC.translate("notifications", "System notifications are not available, you need to install python3-notify2."))
+                return
 
-        if self.radioSysNotifs.isChecked():
-            self._desktop_notifications.show("title", "body")
-        else:
-            pass
+            if self.radioSysNotifs.isChecked():
+                self._desktop_notifications.show("title", "body")
+            else:
+                pass
+        except Exception as e:
+            print(self.LOG_TAG + "exception testing notifications:", e)
+        finally:
+            self.cmdTestNotifs.setEnabled(True)
